@@ -37,13 +37,6 @@
 #include <netinet/in.h>
 #include <sys/un.h>
 
-#if HAVE_VFORK_H
-# include <vfork.h>
-#endif
-#if HAVE_WORKING_FORK
-# define vfork fork
-#endif
-
 #include "lldp-tlv.h"
 #if defined (ENABLE_CDP) || defined (ENABLE_FDP)
 #  include "protocols/cdp.h"
@@ -114,6 +107,7 @@ void	 lldpd_send(struct lldpd_hardware *);
 void	 lldpd_loop(struct lldpd *);
 int	 lldpd_main(int, char **, char **);
 void	 lldpd_update_localports(struct lldpd *);
+void	 lldpd_update_localchassis(struct lldpd *);
 void	 lldpd_cleanup(struct lldpd *);
 
 /* frame.c */
@@ -185,6 +179,7 @@ void		 agent_notify(struct lldpd_hardware *, int, struct lldpd_port *);
 #ifdef ENABLE_PRIVSEP
 /* agent_priv.c */
 void		 agent_priv_register_domain(void);
+const char	*agent_default_agentx_socket(void);
 #endif
 
 /* client.c */
